#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Last updated on Sep 26, 2021
This script loads data
from the RAW_DATA folder, 
and runs all scripts to produce results.
Results are saved in the RESULTS folder. 

This script generates the following results:
1. Age-specific Mortality Rates (ASMR)
2. 1-year Life Expectancy
@author: Woosub Shin (woosubs@umich.edu)
"""

import os
import pandas as pd
os.chdir('/Users/woosubs/Desktop/Research/SouthAfricaHIV/archive_replicability/SCRIPT')
import constants as cn
import rev_calculate_le as cl
import rev_process_thembisa as pt

RAW_DATA_DIR = os.path.join(os.getcwd(), '../RAW_DATA')
PROCESSED_DATA_DIR = os.path.join(os.getcwd(), '../PROCESSED_DATA')
RESULT_DIR = os.path.join(os.getcwd(), '../RESULT')

#######################################################
### 1. Calculating age-specific mortality rates (ASMR)
#######################################################
base_df = pd.read_csv(os.path.join(RAW_DATA_DIR, 'thembisa2000.csv'), index_col=0)
baseline_pop_dist = pt.calculatePopulationDistribution(base_df)
all_cause_asmr = pd.Series(index=cn.EXTENDED_YEARS, dtype='float64')
hiv_aids_asmr = pd.Series(index=cn.EXTENDED_YEARS, dtype='float64')
no_hiv_aids_asmr = pd.Series(index=cn.EXTENDED_YEARS, dtype='float64')
for yr in cn.EXTENDED_YEARS:
  one_df = pd.read_csv(os.path.join(RAW_DATA_DIR, 'thembisa'+str(yr)+'.csv'), index_col=0)
  one_allcause_asmr = pt.getASMR(one_df, baseline_pop_dist, death_type=cn.ALLCAUSE)
  one_hiv_aids_asmr = pt.getASMR(one_df, baseline_pop_dist, death_type=cn.HIVAIDS)
  one_no_hiv_aids_asmr = pt.getASMR(one_df, baseline_pop_dist, death_type=cn.NONHIVAIDS)
  all_cause_asmr[yr] = one_allcause_asmr
  hiv_aids_asmr[yr] = one_hiv_aids_asmr
  no_hiv_aids_asmr[yr] = one_no_hiv_aids_asmr
full_asmr = pd.concat([all_cause_asmr, hiv_aids_asmr, no_hiv_aids_asmr], axis=1)
full_asmr.columns = [cn.ALLCAUSE, cn.HIVAIDS, cn.NONHIVAIDS]
full_asmr.to_csv(os.path.join(RESULT_DIR, 'age_specific_mortality_rates.csv'))
print("All-cause mortality rates were calculated and saved.")
#####################################################


#######################################################
### 2. Calculating 1-year life expectancy
###    for men, women, and both sexes
#######################################################
# I. Actual life expectancy
actual_le_men_df = pd.DataFrame(index=cn.AGE_EXTENDED)
actual_le_women_df = pd.DataFrame(index=cn.AGE_EXTENDED)
actual_le_both_df = pd.DataFrame(index=cn.AGE_EXTENDED)
#
for yr in cn.CALCULATE_YEARS:
  one_df = pd.read_csv(os.path.join(RAW_DATA_DIR, 'thembisa'+str(yr)+'.csv'), index_col=0)
  one_nmx_df = cl.calculate_NMX(one_df)
  life_expectancy_men = cl.calculate_life_expectancy(cl.get_sex_specific_nmx(one_nmx_df, woman=0)[cn.ALLCAUSE],
                                                     woman=0, 
                                                     name='le_men',
                                                     get_full=True)
  life_expectancy_women = cl.calculate_life_expectancy(cl.get_sex_specific_nmx(one_nmx_df, woman=1)[cn.ALLCAUSE],
                                                       woman=1, 
                                                       name='le_women',
                                                       get_full=True)
  life_expectancy_both = cl.calculate_life_expectancy(cl.get_sex_specific_nmx(one_nmx_df, woman=2)[cn.ALLCAUSE],
                                                      woman=2, 
                                                      name='le_both',
                                                      get_full=True)
  actual_le_men_df[yr] = life_expectancy_men
  actual_le_women_df[yr] = life_expectancy_women
  actual_le_both_df[yr] = life_expectancy_both
# save actual life expectancy
actual_le_men_df.to_csv(os.path.join(RESULT_DIR, 'actual_le_men.csv'))
actual_le_women_df.to_csv(os.path.join(RESULT_DIR, 'actual_le_women.csv'))
actual_le_both_df.to_csv(os.path.join(RESULT_DIR, 'actual_le_both.csv'))
#######################################################
# II. Best-case scenario 
# Same as above, except it uses cn.NONHIVAIDS mortality (allcause - HIV mortality)
best_case_le_men_df = pd.DataFrame(index=cn.AGE_EXTENDED)
best_case_le_women_df = pd.DataFrame(index=cn.AGE_EXTENDED)
best_case_le_both_df = pd.DataFrame(index=cn.AGE_EXTENDED)
#
for yr in cn.CALCULATE_YEARS:
  one_df = pd.read_csv(os.path.join(RAW_DATA_DIR, 'thembisa'+str(yr)+'.csv'), index_col=0)
  one_nmx_df = cl.calculate_NMX(one_df)
  life_expectancy_men = cl.calculate_life_expectancy(cl.get_sex_specific_nmx(one_nmx_df, woman=0)[cn.NONHIVAIDS],
                                                     woman=0, 
                                                     name='le_men',
                                                     get_full=True)
  life_expectancy_women = cl.calculate_life_expectancy(cl.get_sex_specific_nmx(one_nmx_df, woman=1)[cn.NONHIVAIDS],
                                                       woman=1, 
                                                       name='le_women', get_full=True)
  life_expectancy_both = cl.calculate_life_expectancy(cl.get_sex_specific_nmx(one_nmx_df, woman=2)[cn.NONHIVAIDS],
                                                      woman=2, 
                                                      name='le_both',
                                                      get_full=True)
  best_case_le_men_df[yr] = life_expectancy_men
  best_case_le_women_df[yr] = life_expectancy_women
  best_case_le_both_df[yr] = life_expectancy_both
# save best-case life expectancy
best_case_le_men_df.to_csv(os.path.join(RESULT_DIR, 'best_case_le_men.csv'))
best_case_le_women_df.to_csv(os.path.join(RESULT_DIR, 'best_case_le_women.csv'))
best_case_le_both_df.to_csv(os.path.join(RESULT_DIR, 'best_case_le_both.csv'))

#######################################################
# III. Worst-case scenario
# Uses cn.ALLCAUSE-cn.HIVAIDS+baseline_year[cn.HIVAIDS] 
worst_case_le_men_df = pd.DataFrame(index=cn.AGE_EXTENDED)
worst_case_le_women_df = pd.DataFrame(index=cn.AGE_EXTENDED)
worst_case_le_both_df = pd.DataFrame(index=cn.AGE_EXTENDED)
# collect baseline information
baseline_df = pd.read_csv(os.path.join(RAW_DATA_DIR, 'thembisa2006.csv'), index_col=0)
baseline_nmx_df = cl.calculate_NMX(baseline_df)
base_nmx_men = cl.get_sex_specific_nmx(baseline_nmx_df, woman=0)
base_nmx_women = cl.get_sex_specific_nmx(baseline_nmx_df, woman=1)
base_nmx_both = cl.get_sex_specific_nmx(baseline_nmx_df, woman=2)
#
for yr in cn.CALCULATE_YEARS:
  if yr != 2006:
    one_df = pd.read_csv(os.path.join(RAW_DATA_DIR, 'thembisa'+str(yr)+'.csv'), index_col=0)
    one_nmx_df = cl.calculate_NMX(one_df)
  else:
    one_df = baseline_df
    one_nmx_df = baseline_nmx_df
  #
  ss_nmx = cl.get_sex_specific_nmx(one_nmx_df, woman=0)
  life_expectancy_men = cl.calculate_life_expectancy(ss_nmx[cn.ALLCAUSE]-ss_nmx[cn.HIVAIDS]+base_nmx_men[cn.HIVAIDS],
                                                     woman=0, 
                                                     name='le_men',
                                                     get_full=True)
  ss_nmx = cl.get_sex_specific_nmx(one_nmx_df, woman=1)
  life_expectancy_women = cl.calculate_life_expectancy(ss_nmx[cn.ALLCAUSE]-ss_nmx[cn.HIVAIDS]+base_nmx_women[cn.HIVAIDS],
                                                       woman=1, 
                                                       name='le_women',
                                                       get_full=True)
  ss_nmx = cl.get_sex_specific_nmx(one_nmx_df, woman=2)
  life_expectancy_both = cl.calculate_life_expectancy(ss_nmx[cn.ALLCAUSE]-ss_nmx[cn.HIVAIDS]+base_nmx_both[cn.HIVAIDS],
                                                      woman=2, 
                                                      name='le_both',
                                                      get_full=True)
  worst_case_le_men_df[yr] = life_expectancy_men
  worst_case_le_women_df[yr] = life_expectancy_women
  worst_case_le_both_df[yr] = life_expectancy_both
worst_case_le_men_df.to_csv(os.path.join(RESULT_DIR, 'worst_case_le_men.csv'))
worst_case_le_women_df.to_csv(os.path.join(RESULT_DIR, 'worst_case_le_women.csv'))
worst_case_le_both_df.to_csv(os.path.join(RESULT_DIR, 'worst_case_le_both.csv'))
#####################################################
print("Life expectancies were calculated and saved.")

