"""
rev_calculate_le.py
This program uses age-specific mortality data and
estimates nax, then calculates periodic life table

Difference from the original version is that
this one calculates everything for one-year. 
"""
import numpy as np
import pandas as pd

import constants as cn


def calculate_NMX(input_df):
    """
    This function estimates nmx
    using the parsed data,
    which contains number of deaths.
    :param Pandas.DataFrame input_df:
    :return Pandas.DataFrame:
    """
    pass
    input_df[cn.AIDS_DEATH_MEN] = input_df[cn.ALLCAUSE_DEATH_MEN] - \
                                    input_df[cn.NONAIDS_DEATH_MEN]
    input_df[cn.AIDS_DEATH_WOMEN] = input_df[cn.ALLCAUSE_DEATH_WOMEN] - \
                                    input_df[cn.NONAIDS_DEATH_WOMEN]
    nmx_df = pd.DataFrame(index=cn.EXTENDED_AGEGROUP)
    # HIV/AIDS nmx
    nmx_df[cn.NMX_AIDS_MEN] = input_df[cn.AIDS_DEATH_MEN] / input_df[cn.POPULATION_MEN]
    nmx_df[cn.NMX_AIDS_WOMEN] = input_df[cn.AIDS_DEATH_WOMEN] / input_df[cn.POPULATION_WOMEN]
    nmx_df[cn.NMX_AIDS_BOTH] = (input_df[cn.AIDS_DEATH_MEN] + input_df[cn.AIDS_DEATH_WOMEN]) / \
                                (input_df[cn.POPULATION_MEN] + input_df[cn.POPULATION_WOMEN])
    # No-HIV/AIDS nmx
    nmx_df[cn.NMX_NONAIDS_MEN] = input_df[cn.NONAIDS_DEATH_MEN] / input_df[cn.POPULATION_MEN]
    nmx_df[cn.NMX_NONAIDS_WOMEN] = input_df[cn.NONAIDS_DEATH_WOMEN] / input_df[cn.POPULATION_WOMEN]
    nmx_df[cn.NMX_NONAIDS_BOTH] = (input_df[cn.NONAIDS_DEATH_MEN] + input_df[cn.NONAIDS_DEATH_WOMEN]) / \
                                (input_df[cn.POPULATION_MEN] + input_df[cn.POPULATION_WOMEN])
    # All cause nmx
    nmx_df[cn.NMX_ALLCAUSE_MEN] = input_df[cn.ALLCAUSE_DEATH_MEN] / input_df[cn.POPULATION_MEN]
    nmx_df[cn.NMX_ALLCAUSE_WOMEN] = input_df[cn.ALLCAUSE_DEATH_WOMEN] / input_df[cn.POPULATION_WOMEN]
    nmx_df[cn.NMX_ALLCAUSE_BOTH] = (input_df[cn.ALLCAUSE_DEATH_MEN] + input_df[cn.ALLCAUSE_DEATH_WOMEN]) / \
                                (input_df[cn.POPULATION_MEN] + input_df[cn.POPULATION_WOMEN])    
    return nmx_df

def get_sex_specific_nmx(input_nmx_df, woman):
    """
    Subsets nmx_df based on sex.
    (men, women, both)
    :param Pandas.DataFrame input_nmx_df:
    :param int woman: 0=men, 1=women, 2=both sexes
    :return Pandas.DataFrame:
    """
    if woman==0:
      res_df = input_nmx_df[[cn.NMX_AIDS_MEN, cn.NMX_NONAIDS_MEN, cn.NMX_ALLCAUSE_MEN]]
    elif woman==1:
      res_df = input_nmx_df[[cn.NMX_AIDS_WOMEN, cn.NMX_NONAIDS_WOMEN, cn.NMX_ALLCAUSE_WOMEN]]
    elif woman==2:
      res_df = input_nmx_df[[cn.NMX_AIDS_BOTH, cn.NMX_NONAIDS_BOTH, cn.NMX_ALLCAUSE_BOTH]]
    else:
      print("Error! Code doesn't exist")
      return False
    res_df.columns = cn.DEATH_STATUS
    return res_df


# woman: 0 = men, 1 = women, 2 = both sexes
def estimate_nax(nmx_input, woman):
    """
    This function estimates nax using
    age-specific mortality data from
    0, 1, 2, ..., 90+.
    Variable 'woman' is integer such that:
    0 if men, 1 if women, 2 if both sexes.
    nax[1:5] gets one number
    according to Coale and Demeny and divided by 4.
    Both sexes still 0.5 men 0.5 women, as ratio is similar.
    :param pandas.core.series.Series nmx_input:
    :param int woman:
    :return array nax:
    """
    # Initialize nax
    num_agegroups = len(nmx_input)
    nax = np.array([0]*5 + [0.5]*(num_agegroups-5))

    # first, estimate 1a0 and 4a1
    # for men (woman==0)
    if (woman == 0) & (nmx_input[0] >= 0.107):
      nax[0] = 0.330
      nax[1:5] = 1.352/4
    elif (woman == 0) & (nmx_input[0] < 0.107):
      nax[0] = 0.045 + 2.684*nmx_input[0]
      nax[1:5] = (1.651 - 2.816*nmx_input[0])/4

    # for women (woman==1)
    elif (woman == 1) & (nmx_input[0] >= 0.107):
      nax[0] = 0.350
      nax[1:5] = 1.361/4
    elif (woman == 1) & (nmx_input[0] < 0.107):
      nax[0] = 0.053 + 2.800*nmx_input[0]
      nax[1:5] = (1.522 - 1.518*nmx_input[0])/4

    # for both sexes (woman==2), weighted sum of both sexes
    elif (woman == 2) & (nmx_input[0] >= 0.107):
      nax[0] = 0.330*0.5 + 0.350*0.5
      nax[1:5] = (1.352*0.5 + 1.361*0.5)/4
    elif (woman == 2) & (nmx_input[0] < 0.107):
      nax[0] = (0.045 + 2.684*nmx_input[0])*0.5 + (0.053 + 2.800*nmx_input[0])*0.5
      nax[1:5] = ((1.651 - 2.816*nmx_input[0])*0.5 + (1.522 - 1.518*nmx_input[0])*0.5)/4      

    # now we estimate the rest with 3 iterations
    for _ in range(3):
      # update nqx and lx
      # nqx = (n*nmx)/(1+(n-nax)*nmx), here, n=1 so formula is simplified
      nqx = nmx_input / (1 + (1 - nax) * nmx_input)
      nqx.iloc[-1] = 1
      lx = np.array([100000] + [0] * (len(nmx_input)-1))
      for i in range(1, len(nmx_input)):
        lx[i] = lx[i-1] * (1 - nqx[i-1])
      ndx = lx * nqx
      for i in range(5, len(nmx_input)-1):
        nax[i] = 0.5 + (1/24)*((ndx[i+1] - ndx[i-1])/ndx[i])
    # you don't need the last entry as npx = 1 for the last age group
    nax[-1] = np.nan

    return nax 


def calculate_life_table(nmx_input, woman_indic, full_return=False):
    """
    Using the estimates from 'estimate_nax' function,
    creates a life table and returns it
    param DataFrame nmx_input: age-specific mortality rates
    param int women: 0 if men, 1 if women, 2 if both
    param bool full_return: if True, return all age groups
    return: DataFrame life_table
    """
    ##
    num_agegroups = len(nmx_input)
    nax = np.array(estimate_nax(nmx_input, woman_indic))
    # nqx = (n*nmx)/(1+(n-nax)*nmx), here, n=1 so formula is simplified
    nqx = np.array((nmx_input) / (1 + (1 - nax) * nmx_input))
    nqx[-1] = 1.
    npx = 1. - nqx

    lx = np.array([100000] + [0] * (num_agegroups-1))
    for i in range(1, num_agegroups):
      lx[i] = lx[i-1] * (npx[i-1])
    #
    ndx = lx * nqx
    nLx = np.array([0] * num_agegroups)
    Tx = np.array([0] * num_agegroups)
    nLx[num_agegroups-1] = lx[num_agegroups-1] / nmx_input[num_agegroups-1]
    Tx[num_agegroups-1] = nLx[num_agegroups-1]
    for i in range(num_agegroups-2, -1, -1):
      nLx[i] = lx[i+1] + (nax[i] * ndx[i])
      Tx[i] = Tx[i+1] + nLx[i]

    ex = np.round(Tx / lx, 2)
    res_idx = ["at " + str(s) for s in cn.EXTENDED_AGEGROUP]
    res_idx[0] = "at birth"
    res_idx[-1] = "90+"
    # create a life table
    life_table = pd.DataFrame(
      {'nmx': np.array(nmx_input), 'nax': nax, 'nqx': nqx, 'npx': npx, 'lx': lx,
       'ndx': ndx, 'nLx': nLx, 'Tx': Tx, 'ex': ex}, index=res_idx,
        columns=['nmx', 'nax', 'nqx', 'npx', 'lx', 'ndx', 'nLx', 'Tx', 'ex'])
    if full_return:
      return life_table
    else:
      return life_table.loc[cn.AGE_SHORTENED]

def calculate_life_expectancy(nmx_input, woman, name='life_expectancy', get_full=False):
    """
    returns life expectancy of the calculated life table
    input: age-specific mortality rates, sex
    output: life expectancy (data frame)
    """
    calculated_lt = calculate_life_table(nmx_input, woman, get_full)
    life_expectancy = calculated_lt.rename(columns = {'ex': name}, inplace=True)
    return calculated_lt[[calculated_lt.columns[-1]]]
