"""
rev_process_thembisa.py
This script summarizes Thembisa data, 
and calculate mortlaity rates.

It assumes that Themibsa model was
already was run and stored somewhere.  
"""
import numpy as np
import pandas as pd

import constants as cn


def getParseMortalityData(file_path):
  """
  Summarize for baseline year data.
  name of column for number of deaths is required. 
  Imported file is Thembisa model file, 
  with 'Monthly' tab which contains 
  number of deaths and population size for each age group.
  :param str file_path: location and name of file
  :return Pandas.DataFrame: 
  """
  # mort_df: number of deaths for men and women
  # Column indices were manually types
  mort_df = pd.read_excel(file_path,
                    sheet_name=cn.MONTHLY_TAB,
                    skiprows=151, nrows=92, usecols=[0,3,4,5,6])
  # drop one of the duplicated age at 10; we use the second one
  mort_df = mort_df.drop(10, axis=0)
  mort_df.columns = [cn.AGE,
                     cn.NONAIDS_DEATH_MEN, cn.NONAIDS_DEATH_WOMEN,
                     cn.ALLCAUSE_DEATH_MEN, cn.ALLCAUSE_DEATH_WOMEN]
  mort_df.index = mort_df[cn.AGE]
  mort_df = mort_df.drop(cn.AGE, axis=1)
  # pop_df: population size for men and women
  pop_df = pd.read_excel(file_path,
                         sheet_name=cn.MONTHLY_TAB,
                         skiprows=324, nrows=91, usecols=[0,1,2])
  pop_df.columns = [cn.AGE, cn.POPULATION_MEN, cn.POPULATION_WOMEN]
  pop_df.index = pop_df[cn.AGE]
  # drop age column, as it is already used as index of DataFrame
  pop_df = pop_df.drop(cn.AGE, axis=1)
  res_df = pd.concat([mort_df, pop_df], axis=1)
  return res_df


def calculatePopulationDistribution(df, woman=2):
  """
  Calculate the proportion of each age group,
  in order to calculate ASMR
  :param Pandas.DataFrame base_df: baseline data is needed here
  :param int women: 0 for men, 1 for women, 2 for both
  :return Pandas.Series:
  """
  if woman==0:
    num_pop = df[cn.POPULATION_MEN]
  elif woman==1:
    num_pop = df[cn.POPULATION_WOMEN]
  else:
    num_pop = df[cn.POPULATION_MEN] + df[cn.POPULATION_WOMEN]
  dist_pop = num_pop / num_pop.sum()
  result = pd.Series(data=dist_pop, index=df.index, dtype='float64')
  return result


def getASMR(df, pop_dist, death_type=cn.ALLCAUSE):
  """
  Calculate Age-specific Mortality Rates
  for one year (specified as df).
  Name of columns are same as that from 
  'getParseMortalityData'.
  pop_dist specified the age-distribution
  of baseline year (or population) from 0-90.
  Inspired by: https://www.statcan.gc.ca/eng/dai/btd/asr
  :param Pandas.DataFrame df:
  :param Pandas.Series pop_dist:
  :return numpy.float64:
  """
  if death_type==cn.ALLCAUSE:
    numdeath = df[cn.ALLCAUSE_DEATH_MEN] + df[cn.ALLCAUSE_DEATH_WOMEN]
  elif death_type==cn.NONHIVAIDS:
    numdeath = df[cn.NONAIDS_DEATH_MEN] + df[cn.NONAIDS_DEATH_WOMEN]
  elif death_type==cn.HIVAIDS:
    numdeath = df[cn.ALLCAUSE_DEATH_MEN] - df[cn.NONAIDS_DEATH_MEN] + \
    df[cn.ALLCAUSE_DEATH_WOMEN] -  df[cn.NONAIDS_DEATH_WOMEN]
  numpop = df[cn.POPULATION_MEN] + df[cn.POPULATION_WOMEN]
  age_specific_death_rate = numdeath / numpop * cn.STAN_POP_SIZE
  age_specific_standardized_death_rate = (age_specific_death_rate * pop_dist)
  asmr = age_specific_standardized_death_rate.sum()
  return asmr



def getSSASMR(df, pop_dist, woman, death_type=cn.ALLCAUSE):
  """
  Calculate Age-specific Mortality Rates
  for one year (specified as df).
  Name of columns are same as that from 
  'getParseMortalityData'.
  pop_dist specified the age-distribution
  of baseline year (or population) from 0-90.
  Inspired by: https://www.statcan.gc.ca/eng/dai/btd/asr
  :param Pandas.DataFrame df:
  :param int woman: 0 if men, 1 if women
  :param Pandas.Series pop_dist:
  :return numpy.float64:
  """
  if woman==0:
    sdf = df[[cn.NONAIDS_DEATH_MEN, cn.ALLCAUSE_DEATH_MEN, cn.POPULATION_MEN]]
  elif woman==1:
    sdf = df[[cn.NONAIDS_DEATH_WOMEN, cn.ALLCAUSE_DEATH_WOMEN, cn.POPULATION_WOMEN]]
  sdf.columns = [cn.NONHIVAIDS, cn.ALLCAUSE, cn.POP_SIZE]
  sdf = sdf.assign(hiv_aids=np.array(sdf[cn.ALLCAUSE]) - np.array(sdf[cn.NONHIVAIDS]))
  #
  if death_type==cn.ALLCAUSE:
    numdeath = sdf[cn.ALLCAUSE]  
  elif death_type==cn.NONHIVAIDS:
    numdeath = sdf[cn.NONHIVAIDS] 
  elif death_type==cn.HIVAIDS:
    numdeath = sdf[cn.HIVAIDS] 
  #
  numpop = sdf[cn.POP_SIZE]
  age_specific_death_rate = numdeath / numpop * cn.STAN_POP_SIZE
  age_specific_standardized_death_rate = (age_specific_death_rate * pop_dist)
  asmr = age_specific_standardized_death_rate.sum()
  return asmr


























